/*
- This code is created by Muhammad Muddassir Malik
- Animations are not tied to the time interval for simplicity purposes
*/


let sun;                          // variable to hold image of the sun
let earth;                        // variable to hold image of the earth          
let noOfStars = 100;              // number of stars to display
let sizeOfSun = 0.7;              // scaling factor for the image of sun
let stars = [];                   // An Array to hold the x,y coordinates of the stars
let angle = 0;                    // angle for the rotation, initially set 0 and updated in draw

// Dom Variables
let noOfStarsIP;                  // textbox to enter the number of stars for updation

// Record to hold data about the earth and its moon
let planetEarth =  {  
  size : 0.06,                    // scaling factor for the earth's image
  distFromSun: 200,
  rotSpeed : 1,
  revSpeed : 4,
  moonExists : true,
  moonDist : 50,
  moonRotSpeed : 1,
  moonSize : 10,
};


function preload(){
  // loads images for sun and moon. Other planets and moon just have a colour
  sun = loadImage("data/sun.png");
  earth = loadImage("data/earth.png");
}

function createStars(){
  // Randomly generate locations for stars and store them in an array in the form of vector 
  for (let i=0;i<noOfStars;i++){
    stars[i] = createVector(int(random(width)), int(random(height)));
  }
}

function updateNoOfStars (){
  // Takes a user entered number from the textfield and calls createStars() to randomly generate stars
  noOfStars = int(noOfStarsIP.value());
  stars = [];
  createStars();
}

function drawStars(){
  // called from draw, it loops through the array and draw a str as ellipse for each location available
  fill(255, 255, 255);
  noStroke();
  for (let i=0;i<noOfStars;i++){
    ellipse(stars[i].x, stars[i].y, 2, 2);
  }
}

function drawMoon(planet){
  // Given a planet's record, it draws a white moon of that planet using the moon paramters
  fill(255, 255, 255);
  push();
    rotate(angle * planet.moonRotSpeed);
    translate(planet.moonDist, 0);
    ellipse(0, 0, planet.moonSize, planet.moonSize);
  pop();
}

function setup() {
  createCanvas(730, 730);
  createStars();                              // Create initial set of stars (default: 100)
  
  // Setup text field, button and callback function to update number of stars to display in the system
  createP("");
  noOfStarsIP = createInput(noOfStars);
  noOfStarsBT = createButton('submit');
  noOfStarsBT.mousePressed(updateNoOfStars);
    
  // All are shapes and images are drawn using CENTER mode and all angles are in degrees
  rectMode(CENTER);
  angleMode(DEGREES);
  imageMode(CENTER);
}

function draw() {
  background('black');
  drawStars();                            // Draw the stars from the array generated through createStars()
  
  fill(255, 0, 0);                        
  angle += 0.25;                          // angle incremented by 0.25 for each call to render
  
  // Translate to the center of the canvas and draw sun, planets and moons
  translate(width/2, height/2);            
  push();                              
    push();
      scale(sizeOfSun);                 
      image(sun, 0, 0);              
    pop();
    
    rotate(angle * planetEarth.rotSpeed);
    translate(planetEarth.distFromSun, 0);
  
    push();
      rotate(-angle * planetEarth.revSpeed);
      scale(planetEarth.size);
      image(earth, 0, 0);
    pop();
  
    if (planetEarth.moonExists == true)
      drawMoon(planetEarth);              // draw moon if it exists
   
  pop();
}
