/*
- This code is created by Muhammad Muddassir Malik
*/

let sun;                          // variable to hold image of the sun
let earth;                        // variable to hold image of the earth
let noOfStars = 100;              // number of stars to display
let sizeOfSun = 0.7;              // scaling factor for the image of sun
let stars = [];                   // An Array to hold the x,y coordinates of the stars
let angle = 0;                    // angle for the rotation, initially set 0 and updated in draw

// Dom Variables
let noOfStarsIP;                  // textbox to enter the number of stars for updation
let earthDistFromSunSD;           // slider to update earth distance from sun
let earthRotSpeedSD;              // slider to update earth's rotation speed
let earthRevSpeedSD;              // slider to update earth's revolution speed
let moonDistSD;                   // slider to update moon's distance to earth
let moonRotSpeedSD;               // slider to update speed of moon's rotation

let addPlanetIP;                  // textbox to enter planet information
let addPlanetBT;                  // button to add a planet information to the system

// Record to hold data about the earth and its moon
let planetEarth =  {
  size : 0.06,                    // scaling factor for the earth's image
  distFromSun: 200,
  rotSpeed : 1,
  revSpeed : 4,
  moonExists : true,
  moonDist : 50,
  moonRotSpeed : 1,
  moonSize : 10,
};

let planets = [];                 // Array to hold planets that user adds (push) or deletes (pop)
let noOfPlanets = 0;              // counter to maintains the number of planets

function preload(){
  // loads images for sun and moon. Other planets and moon just have a colour
  sun = loadImage("data/sun.png");
  earth = loadImage("data/earth.png");
}

function createStars(){
  // Randomly generate locations for stars and store them in an array in the form of vector
  for (let i=0;i<noOfStars;i++){
    stars[i] = createVector(int(random(width)), int(random(height)));
  }
}

function updateNoOfStars (){
  // Takes a user entered number from the textfield and calls createStars() to randomly generate stars
  noOfStars = int(noOfStarsIP.value());
  stars = [];
  createStars();
}

function drawStars(){
  // called from draw, it loops through the array and draw a str as ellipse for each location available
  fill(255, 255, 255);
  noStroke();
  for (let i=0;i<noOfStars;i++){
    ellipse(stars[i].x, stars[i].y, 2, 2);
  }
}

function drawMoon(planet){
  // Given a planet record, it draws a white moon of that planet using the moon paramters
  fill(255, 255, 255);
  push();
    rotate(angle * planet.moonRotSpeed);
    translate(planet.moonDist, 0);
    ellipse(0, 0, planet.moonSize, planet.moonSize);
  pop();
}

function addPlanetToSystem (){
  /*record format tht will be used to store data about each planet and its moons.
    These records are added once for each planet in the array planets
    let planetGeneral = {
      size : 30,
      colour : 'green',
      rotSpeed : 1,
      revSpeed : 4,
      distFromSun: 200,
      moonExists : true,
      moonDist : 50,
      moonRotSpeed : 1,
      moonSize : 10,
    };
    i/p example: 30, green, 3, 4, 300, true, 30, 30, 7
    i/p example: 40, red, 1, 10, 370, false, 0, 0, 0
    i/p example: 20, orange, 10, 2, 80, true, 17, 6, 6
  */
  plString = addPlanetIP.value();            // get user entered string
  pl = splitTokens(plString, ' ,');          // split the string into array
  let pg;                        
  if (noOfPlanets < 9){                // 10 is the total number of planets that can be entered
    pg = {                              // extracting data from array and populating record
      size : int(pl[0]),
      colour : pl[1],
      rotSpeed : int(pl[2]),
      revSpeed : int(pl[3]),
      distFromSun: int(pl[4]),
      moonExists : pl[5] === 'true',
      moonDist : int(pl[6]),
      moonRotSpeed : int(pl[7]),
      moonSize : int(pl[8]),
    };
    addPlanetIP.value("");            // setting text field to empty
    planets.push(pg);                 // Push the record of the planet and moon into array
    noOfPlanets++;                    // increment the number of planets
  }
}

function removeLastPlanetFromSystem (){
  if (noOfPlanets > 0){              // check if there are any planets to avoinf underflowing
    planets.pop();                   // remove the last added planet 
    noOfPlanets--;                   // update the number of planets in the system
  }
}

function drawAPlanet(index){
  // this function is called from the draw for each planet. Data is extracted from the record stored
  fill(planets[index].colour);
  push();
    rotate(angle * planets[index].rotSpeed);
    translate(planets[index].distFromSun, 0);
    push();
      rotate(-angle * planets[index].revSpeed);
      ellipse(0, 0, planets[index].size, planets[index].size);
    pop();
    if (planets[index].moonExists == true)          // Draw if the planet has a moon
      drawMoon(planets[index]);
  pop();
}

function setup() {
  createCanvas(730, 730);
  createStars();                              // Create initial set of stars (default: 100)
  
  // Setup text field, button and callback function to update number of stars to display in the system
  createP("");
  noOfStarsIP = createInput(noOfStars);
  noOfStarsBT = createButton('submit');
  noOfStarsBT.mousePressed(updateNoOfStars);
  
  // setup slider to control earth's distance from sun
  createP("");
  createDiv("Earth's Distance From Sun");
  earthDistFromSunSD = createSlider(50, 300, planetEarth.distFromSun);
  earthDistFromSunSD.style('width', '200px');
  
  // setup slider to control earth's speed of rotation around sun. This value is multiplied with the angle
  createDiv("Earth's Rotation Speed"); // createP("");
  earthRotSpeedSD = createSlider(1, 10, planetEarth.rotSpeed);
  earthRotSpeedSD.style('width', '200px');
 
  // setup slider to control earth's speed of rotation about it's axis
  createDiv("Earth's Revolving Speed");
  earthRevSpeedSD = createSlider(3, 20, planetEarth.revSpeed);
  earthRevSpeedSD.style('width', '200px');
  
  // setup slider to control moon's distance from earth
  createDiv("Moon's Distane From Earth");
  moonDistSD = createSlider(30, 70, planetEarth.moonDist);
  moonDistSD.style('width', '200px');
  
  // setup slider to control moon's rotation
  createDiv("Moon's Rotation Speed");
  moonRotSpeedSD = createSlider(1, 10, planetEarth.moonRotSpeed);
  moonRotSpeedSD.style('width', '200px');
  
  //create textfield, and two button to add and remove planets
  createP("");
  createDiv("Adding a new planet");
  addPlanetIP = createInput("", 100);
  addPlanetIP.size(400);
  addPlanetBT = createButton("Add Planet");
  addPlanetBT.mousePressed(addPlanetToSystem);
  removePlanetBT = createButton("Remove Last Planet");
  removePlanetBT.mousePressed(removeLastPlanetFromSystem);
  
  // All are shapes and images are drawn using CENTER mode and all angles are in degrees
  rectMode(CENTER);
  angleMode(DEGREES);
  imageMode(CENTER);
}

function draw() {
  // Get latest values from the sliders to be used in rendering
  planetEarth.distFromSun = earthDistFromSunSD.value();
  planetEarth.rotSpeed = earthRotSpeedSD.value();
  planetEarth.revSpeed = earthRevSpeedSD.value();
  planetEarth.moonDist = moonDistSD.value();
  planetEarth.moonRotSpeed = moonRotSpeedSD.value();
  
  background('black');
  drawStars();                            // Draw the stars from the array generated through createStars()
  
  fill(255, 0, 0);
  angle += 0.25;                          // angle incremented by 0.25 for each call to render
  
  // Translate to the center of the canvas and draw sun, planets and moons
  translate(width/2, height/2);
  push();
    push();
      scale(sizeOfSun);
      image(sun, 0, 0);
    pop();
    
    rotate(angle * planetEarth.rotSpeed);
    translate(planetEarth.distFromSun, 0);
  
    push();
      rotate(-angle * planetEarth.revSpeed);
      scale(planetEarth.size);
      image(earth, 0, 0);
    pop();
  
    if (planetEarth.moonExists == true)
      drawMoon(planetEarth);              // draw moon if it exists
  pop();
 
  for (let i=0; i<noOfPlanets; i++)
    drawAPlanet(i);                        // Loop to call drawAPlanet for each planet entered by the user
}
